<?php
declare(strict_types=1);

namespace CakeContactUs\Mailer;

use Cake\Core\Configure;
use Cake\Datasource\EntityInterface;
use Cake\Mailer\Mailer;
use Cake\Mailer\Message;

/**
 * User Mailer
 */
class ContactUsFormSubmissionsMailer extends Mailer
{
    /**
     * Send the templated email to the user
     *
     * @param \Cake\Datasource\EntityInterface $user User entity
     * @return void
     */
    protected function confirmation(EntityInterface $contactUsFormSubmission, array $options = [])
    {
        $name = isset($contactUsFormSubmission['name']) ? $contactUsFormSubmission['name'] . ', ' : '';
        // un-hide the token to be able to send it in the email content
        $subject = __d('cake_contact_us', 'We have received your message');


        $this
            ->setTo($contactUsFormSubmission['email'])
            ->setSubject($name . $subject)
            ->setEmailFormat(Message::MESSAGE_BOTH)
            ->setViewVars([
                'contactUsFormSubmission' => $contactUsFormSubmission,
            ]);

        $this->viewBuilder()
            ->setTemplate('CakeContactUs.confirmation');
    }

    /**
     * Send to backoffice to take action
     *
     * @param \Cake\Datasource\EntityInterface $user User entity
     * @return void
     */
    protected function backend(EntityInterface $contactUsFormSubmission, array $options = [])
    {
        $name = isset($contactUsFormSubmission['name']) ? ' by ' . $contactUsFormSubmission['name'] : '';
        // un-hide the token to be able to send it in the email content
        $subject = __d('cake_contact_us', 'Contact Us Form Submitted');


        $this
            ->setTo(Configure::readOrFail('ContactUs.email.backend.to'))
            ->setSubject($subject . $name)
            ->setEmailFormat(Message::MESSAGE_BOTH)
            ->setViewVars([
                'contactUsFormSubmission' => $contactUsFormSubmission,
            ]);

        $this->viewBuilder()
            ->setTemplate('CakeContactUs.backend');
    }
}
